@@ -3,16 +3,14 @@
 Spectrum estimation and modeling methods (1-dimensional, with an energy axis).
 """
 from .core import *
-from .butterfly import *
 from .observation import *
 from .cosmic_ray import *
 from .crab import *
-from .diffuse import *
 from .energy_group import *
 from .flux_point import *
-from .sherpa_chi2asym import *
 from .utils import *
 from .extract import *
 from .simulation import *
 from .fit import *
 from .results import *
+from .sensitivity import *
