@@ -8,12 +8,7 @@ import numpy as np
 from astropy.units import Quantity
 from .array import array_stats_str
 
-__all__ = [
-    'NDDataArray',
-    'DataAxis',
-    'BinnedDataAxis',
-    'sqrt_space',
-]
+__all__ = ["NDDataArray", "DataAxis", "BinnedDataAxis", "sqrt_space"]
 
 
 class NDDataArray(object):
@@ -32,15 +27,14 @@ class NDDataArray(object):
     interp_kwargs : dict
         TODO
     """
+
     default_interp_kwargs = dict(bounds_error=False)
     """Default interpolation kwargs used to initialize the
     `scipy.interpolate.RegularGridInterpolator`.  The interpolation behaviour
     of an individual axis ('log', 'linear') can be passed to the axis on
     initialization."""
 
-    def __init__(self, axes, data=None, meta=None,
-                 interp_kwargs=None):
-
+    def __init__(self, axes, data=None, meta=None, interp_kwargs=None):
         self._axes = axes
         if data is not None:
             self.data = data
@@ -51,11 +45,10 @@ class NDDataArray(object):
         self._regular_grid_interp = None
 
     def __str__(self):
-        """String representation"""
-        ss = 'NDDataArray summary info\n'
+        ss = "NDDataArray summary info\n"
         for axis in self.axes:
             ss += array_stats_str(axis.nodes, axis.name)
-        ss += array_stats_str(self.data, 'Data')
+        ss += array_stats_str(self.data, "Data")
         return ss
 
     @property
@@ -68,7 +61,7 @@ class NDDataArray(object):
         try:
             idx = [_.name for _ in self.axes].index(name)
         except ValueError:
-            raise ValueError('Axis {} not found'.format(name))
+            raise ValueError("Axis {} not found".format(name))
         return self.axes[idx]
 
     @property
@@ -91,17 +84,19 @@ class NDDataArray(object):
         data = Quantity(data)
         dimension = len(data.shape)
         if dimension != self.dim:
-            raise ValueError('Overall dimensions to not match. '
-                             'Data: {}, Hist: {}'.format(dimension, self.dim))
+            raise ValueError(
+                "Overall dimensions to not match. "
+                "Data: {}, Hist: {}".format(dimension, self.dim)
+            )
 
         for dim in np.arange(self.dim):
             axis = self.axes[dim]
             if axis.nbins != data.shape[dim]:
-                msg = 'Data shape does not match in dimension {d}\n'
-                msg += 'Axis {n} : {sa}, Data {sd}'
-                raise ValueError(msg.format(d=dim, n=axis.name,
-                                            sa=axis.nbins,
-                                            sd=data.shape[dim]))
+                msg = "Data shape does not match in dimension {d}\n"
+                msg += "Axis {n} : {sa}, Data {sd}"
+                raise ValueError(
+                    msg.format(d=dim, n=axis.name, sa=axis.nbins, sd=data.shape[dim])
+                )
         self._regular_grid_interp = None
         self._data = data
 
@@ -159,14 +154,23 @@ class NDDataArray(object):
         if kwargs != {}:
             raise ValueError("Input given for unknown axis: {}".format(kwargs))
 
-        if method is None:
-            out = self._eval_regular_grid_interp(values)
-        elif method == 'linear':
-            out = self._eval_regular_grid_interp(values, method='linear')
-        elif method == 'nearest':
-            out = self._eval_regular_grid_interp(values, method='nearest')
+        # This is necessary since np.append does not support the 1D case
+        if self.dim > 1:
+            shapes = np.concatenate([np.shape(_) for _ in values])
         else:
-            raise ValueError('Interpolator {} not available'.format(method))
+            shapes = values[0].shape
+
+        # Flatten in order to support 2D array input
+        values = [_.flatten() for _ in values]
+        points = list(itertools.product(*values))
+
+        if self._regular_grid_interp is None:
+            self._add_regular_grid_interp()
+
+        method = method or self.default_interp_kwargs.get("method", None)
+        res = self._regular_grid_interp(points, method=method, **kwargs)
+
+        out = np.reshape(res, shapes).squeeze()
 
         # Clip interpolated values to be non-negative
         np.clip(out, 0, None, out=out)
@@ -175,25 +179,42 @@ class NDDataArray(object):
 
         return out
 
-    def _eval_regular_grid_interp(self, values, **kwargs):
-        """Evaluate linear interpolator
+    def evaluate_at_coord(self, points, method="linear", **kwargs):
+        """Evaluate NDData Array on set of points.
 
-        Input: list of values to evaluate, in correct units and correct order.
+        TODO: merge with `evaluate`?
+        This method was added to support evaluating on arbitrary arrays
+        of coordinates, not just on the outer product like `evaluate`.
+
+        Parameters
+        ----------
+        points: dict
+            contains the coordinates on which you want to interpolate (axis_name: value)
+        method : str {'linear', 'nearest'}, optional
+            Interpolation method
+        kwargs : dict
+            Keys are the axis names, Values the evaluation points
+
+        Returns
+        -------
+        array : `~astropy.units.Quantity`
+            Interpolated values, axis order is the same as for the NDData array
         """
         if self._regular_grid_interp is None:
             self._add_regular_grid_interp()
 
-        # This is necessary since np.append does not support the 1D case
-        if self.dim > 1:
-            shapes = np.concatenate([np.shape(_) for _ in values])
+        points = tuple(
+            [
+                axis._interp_values(points[axis.name].to(axis.unit).value)
+                for axis in self.axes
+            ]
+        )
+        res = self._regular_grid_interp(points, method=method, **kwargs)
 
-        else:
-            shapes = values[0].shape
-        # Flatten in order to support 2D array input
-        values = [_.flatten() for _ in values]
-        points = list(itertools.product(*values))
-        res = self._regular_grid_interp(points, **kwargs)
-        res = np.reshape(res, shapes).squeeze()
+        # Clip interpolated values to be non-negative
+        np.clip(res, 0, None, out=res)
+        # Attach units to the output
+        res = res * self.data.unit
 
         return res
 
@@ -212,20 +233,24 @@ class NDDataArray(object):
         if interp_kwargs is None:
             interp_kwargs = self.interp_kwargs
         points = [a._interp_nodes() for a in self.axes]
+
         values = self.data.value
 
         # If values contains nan, only setup interpolator in valid range
         if np.isnan(values).any():
             if self.dim > 1:
-                raise NotImplementedError('Data grid contains nan. This is not'
-                                          'supported for arrays dimension > 1')
+                raise NotImplementedError(
+                    "Data grid contains nan. This is not"
+                    "supported for arrays dimension > 1"
+                )
             else:
                 mask = np.isfinite(values)
                 points = [points[0][mask]]
                 values = values[mask]
 
-        self._regular_grid_interp = RegularGridInterpolator(points, values,
-                                                            **interp_kwargs)
+        self._regular_grid_interp = RegularGridInterpolator(
+            points, values, **interp_kwargs
+        )
 
 
 class DataAxis(object):
@@ -245,7 +270,7 @@ class DataAxis(object):
         Interpolation behaviour, default: 'linear'
     """
 
-    def __init__(self, nodes, name='Default', interpolation_mode='linear'):
+    def __init__(self, nodes, name="Default", interpolation_mode="linear"):
         # Need this for subclassing (see BinnedDataAxis)
         if nodes is not None:
             self._data = Quantity(nodes)
@@ -254,10 +279,10 @@ class DataAxis(object):
 
     def __str__(self):
         ss = self.__class__.__name__
-        ss += '\nName: {}'.format(self.name)
-        ss += '\nUnit: {}'.format(self.unit)
-        ss += '\nNodes: {}'.format(self.nbins)
-        ss += '\nInterpolation mode: {}'.format(self.interpolation_mode)
+        ss += "\nName: {}".format(self.name)
+        ss += "\nUnit: {}".format(self.unit)
+        ss += "\nNodes: {}".format(self.nbins)
+        ss += "\nInterpolation mode: {}".format(self.interpolation_mode)
 
         return ss
 
@@ -284,7 +309,7 @@ class DataAxis(object):
         unit : `~astropy.units.UnitBase`, str
             Unit
         """
-        kwargs.setdefault('interpolation_mode', 'log')
+        kwargs.setdefault("interpolation_mode", "log")
 
         if unit is not None:
             vmin = Quantity(vmin, unit)
@@ -311,8 +336,11 @@ class DataAxis(object):
         val = Quantity(val)
 
         if not val.unit.is_equivalent(self.unit):
-            raise ValueError('Units {} and {} do not match'.format(
-                val.unit, self.unit))
+            raise ValueError(
+                "Units mismatch: val.unit = {!r}, self.unit = {!r}".format(
+                    val.unit, self.unit
+                )
+            )
 
         val = val.to(self.nodes.unit)
         val = np.atleast_1d(val)
@@ -340,14 +368,14 @@ class DataAxis(object):
 
     def _interp_nodes(self):
         """Nodes to be used for interpolation"""
-        if self.interpolation_mode == 'log':
+        if self.interpolation_mode == "log":
             return np.log10(self.nodes.value)
         else:
             return self.nodes.value
 
     def _interp_values(self, values):
         """Transform values correctly for interpolation"""
-        if self.interpolation_mode == 'log':
+        if self.interpolation_mode == "log":
             return np.log10(values)
         else:
             return values
@@ -381,8 +409,8 @@ class BinnedDataAxis(DataAxis):
 
     def __str__(self):
         ss = super(BinnedDataAxis, self).__str__()
-        ss += '\nLower bounds {}'.format(self.lo)
-        ss += '\nUpper bounds {}'.format(self.hi)
+        ss += "\nLower bounds {}".format(self.lo)
+        ss += "\nUpper bounds {}".format(self.hi)
 
         return ss
 
@@ -405,7 +433,7 @@ class BinnedDataAxis(DataAxis):
         Depending on the interpolation mode, either log or lin center are
         returned
         """
-        if self.interpolation_mode == 'log':
+        if self.interpolation_mode == "log":
             return self.log_center()
         else:
             return self.lin_center()
